@@ -8,7 +8,7 @@ import vobject
 
 from khard.carddav_object import VCardWrapper
 
-from .helpers import create_test_vcard
+from .helpers import vCard, TestVCardWrapper
 
 
 def _from_file(path):
@@ -20,7 +20,7 @@ def _from_file(path):
 class VcardWrapperInit(unittest.TestCase):
 
     def test_stores_vcard_object_unmodified(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         expected = vcard.serialize()
         wrapper = VCardWrapper(vcard)
         # assert that it is the same object
@@ -29,12 +29,11 @@ class VcardWrapperInit(unittest.TestCase):
         self.assertEqual(wrapper.vcard.serialize(), expected)
 
     def test_warns_about_unsupported_version(self):
-        vcard = create_test_vcard(version="something unsupported")
         with self.assertLogs(level="WARNING"):
-            VCardWrapper(vcard)
+            TestVCardWrapper(version="something unsupported")
 
     def test_warns_about_missing_version_and_sets_it(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         vcard.remove(vcard.version)
         with self.assertLogs(level="WARNING"):
             wrapper = VCardWrapper(vcard)
@@ -44,7 +43,7 @@ class VcardWrapperInit(unittest.TestCase):
 class DeleteVcardObject(unittest.TestCase):
 
     def test_deletes_fields_given_in_upper_case(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         expected = vcard.serialize()
         vcard.add('FOO').value = 'bar'
         wrapper = VCardWrapper(vcard)
@@ -52,7 +51,7 @@ class DeleteVcardObject(unittest.TestCase):
         self.assertEqual(wrapper.vcard.serialize(), expected)
 
     def test_deletes_all_field_occurences(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         expected = vcard.serialize()
         vcard.add('FOO').value = 'bar'
         vcard.add('FOO').value = 'baz'
@@ -61,7 +60,7 @@ class DeleteVcardObject(unittest.TestCase):
         self.assertEqual(wrapper.vcard.serialize(), expected)
 
     def test_deletes_grouped_ablabel_fields(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         expected = vcard.serialize()
         foo = vcard.add('FOO')
         foo.value = 'bar'
@@ -74,7 +73,7 @@ class DeleteVcardObject(unittest.TestCase):
         self.assertEqual(wrapper.vcard.serialize(), expected)
 
     def test_keeps_other_fields(self):
-        vcard = create_test_vcard(foo='bar')
+        vcard = vCard(foo='bar')
         expected = vcard.serialize()
         vcard.add('BAR').value = 'baz'
         wrapper = VCardWrapper(vcard)
@@ -82,7 +81,7 @@ class DeleteVcardObject(unittest.TestCase):
         self.assertEqual(wrapper.vcard.serialize(), expected)
 
     def test_does_not_fail_on_non_existing_field_name(self):
-        vcard = create_test_vcard(foo='bar')
+        vcard = vCard(foo='bar')
         expected = vcard.serialize()
         wrapper = VCardWrapper(vcard)
         wrapper._delete_vcard_object('BAR')
@@ -92,23 +91,21 @@ class DeleteVcardObject(unittest.TestCase):
 class BirthdayLikeAttributes(unittest.TestCase):
 
     def test_birthday_supports_setting_date_objects(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         date = datetime.datetime(2018, 2, 1)
         wrapper.birthday = date
         wrapper.vcard.validate()
         self.assertEqual(wrapper.birthday, date)
 
     def test_birthday_supports_setting_datetime_objects(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         date = datetime.datetime(2018, 2, 1, 19, 29, 31)
         wrapper.birthday = date
         wrapper.vcard.validate()
         self.assertEqual(wrapper.birthday, date)
 
     def test_birthday_supports_setting_text_values_for_v4(self):
-        vcard = create_test_vcard(version="4.0")
+        vcard = vCard(version="4.0")
         wrapper = VCardWrapper(vcard, "4.0")
         date = 'some time yesterday'
         wrapper.birthday = date
@@ -116,31 +113,28 @@ class BirthdayLikeAttributes(unittest.TestCase):
         self.assertEqual(wrapper.birthday, date)
 
     def test_birthday_does_not_support_setting_text_values_for_v3(self):
-        vcard = create_test_vcard(version="3.0")
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper(version="3.0")
         with self.assertLogs(level='WARNING'):
             wrapper.birthday = 'some time yesterday'
         wrapper.vcard.validate()
         self.assertIsNone(wrapper.birthday)
 
     def test_anniversary_supports_setting_date_objects(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         date = datetime.datetime(2018, 2, 1)
         wrapper.anniversary = date
         wrapper.vcard.validate()
         self.assertEqual(wrapper.anniversary, date)
 
     def test_anniversary_supports_setting_datetime_objects(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         date = datetime.datetime(2018, 2, 1, 19, 29, 31)
         wrapper.anniversary = date
         wrapper.vcard.validate()
         self.assertEqual(wrapper.anniversary, date)
 
     def test_anniversary_supports_setting_text_values_for_v4(self):
-        vcard = create_test_vcard(version="4.0")
+        vcard = vCard(version="4.0")
         wrapper = VCardWrapper(vcard, "4.0")
         date = 'some time yesterday'
         wrapper.anniversary = date
@@ -148,8 +142,7 @@ class BirthdayLikeAttributes(unittest.TestCase):
         self.assertEqual(wrapper.anniversary, date)
 
     def test_anniversary_does_not_support_setting_text_values_for_v3(self):
-        vcard = create_test_vcard(version="3.0")
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper(version="3.0")
         with self.assertLogs(level='WARNING'):
             wrapper.birthday = 'some time yesterday'
         wrapper.vcard.validate()
@@ -159,29 +152,27 @@ class BirthdayLikeAttributes(unittest.TestCase):
 class NameAttributes(unittest.TestCase):
 
     def test_fn_can_be_set_with_a_string(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         wrapper = VCardWrapper(vcard)
         wrapper.formatted_name = 'foo bar'
         self.assertEqual(vcard.fn.value, 'foo bar')
 
     def test_only_one_fn_will_be_stored(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         wrapper = VCardWrapper(vcard)
         wrapper.formatted_name = 'foo bar'
         self.assertEqual(len(vcard.contents['fn']), 1)
 
     def test_fn_is_returned_as_string(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         self.assertIsInstance(wrapper.formatted_name, str)
 
     def test_fn_is_used_as_string_representation(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         self.assertEqual(str(wrapper), wrapper.formatted_name)
 
     def test_name_can_be_set_with_empty_strings(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         wrapper = VCardWrapper(vcard)
         wrapper._add_name('', '', '', '', '')
         self.assertEqual(vcard.serialize(),
@@ -192,7 +183,7 @@ class NameAttributes(unittest.TestCase):
                          'END:VCARD\r\n')
 
     def test_name_can_be_set_with_empty_lists(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         wrapper = VCardWrapper(vcard)
         wrapper._add_name([], [], [], [], [])
         self.assertEqual(vcard.serialize(),
@@ -203,7 +194,7 @@ class NameAttributes(unittest.TestCase):
                          'END:VCARD\r\n')
 
     def test_name_can_be_set_with_lists_of_empty_strings(self):
-        vcard = create_test_vcard()
+        vcard = vCard()
         wrapper = VCardWrapper(vcard)
         wrapper._add_name(['', ''], ['', ''], ['', ''], ['', ''], ['', ''])
         self.assertEqual(vcard.serialize(),
@@ -214,26 +205,22 @@ class NameAttributes(unittest.TestCase):
                          'END:VCARD\r\n')
 
     def test_get_first_name_last_name_retunrs_fn_if_no_name_present(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         self.assertEqual(wrapper.get_first_name_last_name(), 'Test vCard')
 
     def test_get_first_name_last_name_with_simple_name(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_name('', 'given', '', 'family', '')
         self.assertEqual(wrapper.get_first_name_last_name(), "given family")
 
     def test_get_first_name_last_name_with_all_name_fields(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_name('prefix', 'given', 'additional', 'family', 'suffix')
         self.assertEqual(wrapper.get_first_name_last_name(),
                          'given additional family')
 
     def test_get_first_name_last_name_with_complex_name(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_name(['prefix1', 'prefix2'], ['given1', 'given2'],
                           ['additional1', 'additional2'],
                           ['family1', 'family2'], ['suffix1', 'suffix2'])
@@ -241,26 +228,22 @@ class NameAttributes(unittest.TestCase):
                          'additional1 additional2 family1 family2')
 
     def test_get_last_name_first_name_retunrs_fn_if_no_name_present(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         self.assertEqual(wrapper.get_last_name_first_name(), 'Test vCard')
 
     def test_get_last_name_first_name_with_simple_name(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_name('', 'given', '', 'family', '')
         self.assertEqual(wrapper.get_last_name_first_name(), "family, given")
 
     def test_get_last_name_first_name_with_all_name_fields(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_name('prefix', 'given', 'additional', 'family', 'suffix')
         self.assertEqual(wrapper.get_last_name_first_name(),
                          'family, given additional')
 
     def test_get_last_name_first_name_with_complex_name(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_name(['prefix1', 'prefix2'], ['given1', 'given2'],
                           ['additional1', 'additional2'],
                           ['family1', 'family2'], ['suffix1', 'suffix2'])
@@ -271,21 +254,18 @@ class NameAttributes(unittest.TestCase):
 class TypedProperties(unittest.TestCase):
 
     def test_adding_a_simple_phone_number(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_phone_number('home', '0123456789')
         self.assertDictEqual(wrapper.phone_numbers, {'home': ['0123456789']})
 
     def test_adding_a_custom_type_phone_number(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_phone_number('custom_type', '0123456789')
         self.assertDictEqual(wrapper.phone_numbers,
                              {'custom_type': ['0123456789']})
 
     def test_adding_multible_phone_number(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_phone_number('work', '0987654321')
         wrapper._add_phone_number('home', '0123456789')
         wrapper._add_phone_number('home', '0112233445')
@@ -295,8 +275,7 @@ class TypedProperties(unittest.TestCase):
             {'home': ['0112233445', '0123456789'], 'work': ['0987654321']})
 
     def test_adding_preferred_phone_number(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_phone_number('home', '0123456789')
         wrapper._add_phone_number('pref,home', '0987654321')
         self.assertDictEqual(
@@ -304,21 +283,18 @@ class TypedProperties(unittest.TestCase):
                                     'home, pref': ['0987654321']})
 
     def test_adding_a_simple_email(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper.add_email('home', 'foo@bar.net')
         self.assertDictEqual(wrapper.emails, {'home': ['foo@bar.net']})
 
     def test_adding_a_custom_type_emails(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper.add_email('custom_type', 'foo@bar.net')
         self.assertDictEqual(wrapper.emails,
                              {'custom_type': ['foo@bar.net']})
 
     def test_adding_multible_emails(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper.add_email('work', 'foo@bar.net')
         wrapper.add_email('home', 'foo@baz.net')
         wrapper.add_email('home', 'baz@baz.net')
@@ -328,16 +304,14 @@ class TypedProperties(unittest.TestCase):
             {'home': ['baz@baz.net', 'foo@baz.net'], 'work': ['foo@bar.net']})
 
     def test_adding_preferred_emails(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper.add_email('home', 'foo@bar.net')
         wrapper.add_email('pref,home', 'foo@baz.net')
         self.assertDictEqual(wrapper.emails, {'home': ['foo@bar.net'],
                                               'home, pref': ['foo@baz.net']})
 
     def test_adding_a_simple_address(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         components = ('box', 'extended', 'street', 'code', 'city', 'region',
                       'country')
         wrapper._add_post_address('home', *components)
@@ -345,8 +319,7 @@ class TypedProperties(unittest.TestCase):
         self.assertDictEqual(wrapper.post_addresses, {'home': [expected]})
 
     def test_adding_a_custom_type_address(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         components = ('box', 'extended', 'street', 'code', 'city', 'region',
                       'country')
         wrapper._add_post_address('custom_type', *components)
@@ -355,8 +328,7 @@ class TypedProperties(unittest.TestCase):
                              {'custom_type': [expected]})
 
     def test_adding_multible_addresses(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         components = ('box', 'extended', 'street', 'code', 'city', 'region',
                       'country')
         wrapper._add_post_address('work', *['work ' + c for c in components])
@@ -371,8 +343,7 @@ class TypedProperties(unittest.TestCase):
                               'work': [expected_work]})
 
     def test_adding_preferred_address(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         components = ('box', 'extended', 'street', 'code', 'city', 'region',
                       'country')
         wrapper._add_post_address('home', *['home1 ' + c for c in components])
@@ -390,8 +361,7 @@ class OtherProperties(unittest.TestCase):
 
     def test_setting_and_getting_organisations(self):
         # also test that organisations are returned in sorted order
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         org1 = ["Org", "Sub1", "Sub2"]
         org2 = ["Org2", "Sub3"]
         org3 = ["Foo", "Bar", "Baz"]
@@ -401,54 +371,46 @@ class OtherProperties(unittest.TestCase):
         self.assertListEqual(wrapper.organisations, [org3, org1, org2])
 
     def test_setting_org_in_different_ways_for_refactoring(self):
-        vcard1 = create_test_vcard()
-        vcard2 = create_test_vcard()
-        wrapper1 = VCardWrapper(vcard1)
-        wrapper2 = VCardWrapper(vcard2)
+        wrapper1 = TestVCardWrapper()
+        wrapper2 = TestVCardWrapper()
         wrapper1._add_organisation('foo')
         wrapper2._add_organisation(['foo'])
         self.assertEqual(wrapper1.organisations, wrapper2.organisations)
 
     def test_setting_and_getting_titles(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_title('Foo')
         wrapper._add_title('Bar')
         self.assertListEqual(wrapper.titles, ['Bar', 'Foo'])
 
     def test_setting_and_getting_roles(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_role('Foo')
         wrapper._add_role('Bar')
         self.assertListEqual(wrapper.roles, ['Bar', 'Foo'])
 
     def test_setting_and_getting_nicks(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_nickname('Foo')
         wrapper._add_nickname('Bar')
         self.assertListEqual(wrapper.nicknames, ['Bar', 'Foo'])
 
     def test_setting_and_getting_notes(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_note('First long note')
         wrapper._add_note('Second long note\nwith newline')
         self.assertListEqual(wrapper.notes, ['First long note',
                              'Second long note\nwith newline'])
 
     def test_setting_and_getting_webpages(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_webpage('https://github.com/scheibler/khard')
         wrapper._add_webpage('http://example.com')
         self.assertListEqual(wrapper.webpages, ['http://example.com',
                              'https://github.com/scheibler/khard'])
 
     def test_setting_and_getting_categories(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_category(["rfc", "address book"])
         wrapper._add_category(["coding", "open source"])
         self.assertListEqual(wrapper.categories,
@@ -459,8 +421,7 @@ class OtherProperties(unittest.TestCase):
 class ABLabels(unittest.TestCase):
 
     def test_setting_and_getting_webpage_ablabel(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._add_webpage({'github': 'https://github.com/scheibler/khard'})
         wrapper._add_webpage('http://example.com')
         self.assertListEqual(wrapper.webpages, [
@@ -472,8 +433,7 @@ class ABLabels(unittest.TestCase):
         self.assertListEqual(vcard.organisations, [{'Work': ['Test Inc']}])
 
     def test_setting_fn_from_labelled_org(self):
-        vcard = create_test_vcard()
-        wrapper = VCardWrapper(vcard)
+        wrapper = TestVCardWrapper()
         wrapper._delete_vcard_object("FN")
         wrapper._add_organisation({'Work': ['Test Inc']})
         self.assertEqual(wrapper.formatted_name, 'Test Inc')
